/* Copyright (c) 2002-2004 Sam Trenholme
 *
 * TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This software is provided 'as is' with no guarantees of correctness or
 * fitness for purpose.
 */

#include "../MaraDns.h"

/* Stuff needed so that tcp stuff compiles w/o MaraDNS.c */

int log_level = 1;

ipv4pair *bind_addr_list = 0;

/* Obtain a bind address from the kvars.  
 * Input: void (we get the data from the kvars)
 * Output: an ipv4pair containing the bind address. */
ipv4pair *libtcp_get_bind_address() {
	js_string *kvar_query, *kvar_value;

	ipv4pair *out;

	int c;

	if((kvar_query = js_create(256,1)) == 0) {
		return 0;
	}

	if(js_qstr2js(kvar_query,"bind_address") == JS_ERROR) {
		js_destroy(kvar_query);
		return 0;
	}

	if((kvar_value = js_create(256,1)) == 0) {
		js_destroy(kvar_query);
		return 0;
	}

	if(read_kvar(kvar_query,kvar_value) == JS_ERROR) {
		js_destroy(kvar_query);
		js_destroy(kvar_value);
		return 0;
	}

	if((out = js_alloc(sizeof(ipv4pair),512)) == 0) {
		js_destroy(kvar_query);
		js_destroy(kvar_value);
		return 0;
	}

	for(c = 0 ; c < 512 ; c++) {
		out[c].ip = 0xffffffff;
	}

	if(make_ip_acl(kvar_value,out,500,0) == JS_ERROR) {
		js_destroy(kvar_query);
		js_destroy(kvar_value);
		js_dealloc(out);
		return 0;
	}

	js_destroy(kvar_query);
	js_destroy(kvar_value);
	return out;

}

/* Create a bind address list 
 * Input: None
 * Output: JS_SUCCESS on success; JS_ERROR on error */

int libtcp_create_bind_addrs() {
	bind_addr_list = libtcp_get_bind_address();
	if(bind_addr_list == 0) {
		return JS_ERROR;
	}
	return JS_SUCCESS;
}

/* Show them the bind address list
 * Input: None
 * Output: The bind address list
 */

/* TO DO: remove this from MaraDNS.c and have MaraDNS.c call this routine */

ipv4pair *get_bind_addr_list() {
	return bind_addr_list;
}

/* Convert a domain-name query in to its lower-case equivalent
 * Input: Pointer to the js string object with the query
 * Output: JS_ERROR on error, JS_SUCCESS on sucess, 0 on
 * success if no change was made to the string */

/* TO DO: remove this from MaraDNS.c and have MaraDNS.c call this routine */

int fold_case(js_string *js) {
	int counter = 0;
	int ret = 0;

	if(js->max_count <= js->unit_count) {
		return JS_ERROR;
	}
	if(js->unit_size != 1) {
		return JS_ERROR;
	}
	if(js->unit_count < 2) {
		return JS_ERROR;
	}
	while(counter + 2 < js->unit_count) {
		/* Since A-Z never happen in a domain length label, we can 
		 * speed things up a bit */
		if(*(js->string + counter) >= 'A' &&
				*(js->string + counter) <= 'Z') {
			*(js->string + counter) += 32;
			ret = 1;
		}
		counter++;
	}

	return ret;
}

